@@ -16,79 +16,41 @@
 
 """InterRepository operations."""
 
-from io import BytesIO
 import itertools
+from typing import Callable, Dict, Tuple, Optional
 
-from dulwich.errors import (
-    NotCommitError,
-    )
-from dulwich.object_store import (
-    ObjectStoreGraphWalker,
-    )
-from dulwich.protocol import (
-    CAPABILITY_THIN_PACK,
-    ZERO_SHA,
-    )
-from dulwich.refs import (
-    ANNOTATED_TAG_SUFFIX,
-    SYMREF,
-    )
+from dulwich.errors import NotCommitError
+from dulwich.objects import ObjectID
+from dulwich.object_store import ObjectStoreGraphWalker
+from dulwich.pack import PACK_SPOOL_FILE_MAX_SIZE
+from dulwich.protocol import CAPABILITY_THIN_PACK, ZERO_SHA
+from dulwich.refs import SYMREF
+try:
+    from dulwich.refs import PEELED_TAG_SUFFIX
+except ImportError:  # dulwich < 0.21.3
+    from dulwich.refs import ANNOTATED_TAG_SUFFIX as PEELED_TAG_SUFFIX
 from dulwich.walk import Walker
 
-from ..errors import (
-    DivergedBranches,
-    FetchLimitUnsupported,
-    InvalidRevisionId,
-    LossyPushToSameVCS,
-    NoRoundtrippingSupport,
-    NoSuchRevision,
-    )
-from ..repository import (
-    FetchResult,
-    InterRepository,
-    )
-from ..revision import (
-    NULL_REVISION,
-    )
-from .. import (
-    config,
-    trace,
-    ui,
-    )
+from .. import config, trace, ui
+from ..errors import (DivergedBranches, FetchLimitUnsupported,
+                      InvalidRevisionId, LossyPushToSameVCS,
+                      NoRoundtrippingSupport, NoSuchRevision)
+from ..repository import FetchResult, InterRepository, AbstractSearchResult
+from ..revision import NULL_REVISION, RevisionID
+from .errors import NoPushSupport
+from .fetch import DetermineWantsRecorder, import_git_objects
+from .mapping import needs_roundtripping
+from .object_store import get_object_store
+from .push import MissingObjectsIterator, remote_divergence
+from .refs import is_tag, ref_to_tag_name
+from .remote import RemoteGitError, RemoteGitRepository
+from .repository import GitRepository, GitRepositoryFormat, LocalGitRepository
+from .unpeel_map import UnpeelMap
 
-from .errors import (
-    NoPushSupport,
-    )
-from .fetch import (
-    import_git_objects,
-    DetermineWantsRecorder,
-    )
-from .mapping import (
-    needs_roundtripping,
-    )
-from .object_store import (
-    get_object_store,
-    )
-from .push import (
-    MissingObjectsIterator,
-    remote_divergence,
-    )
-from .refs import (
-    is_tag,
-    ref_to_tag_name,
-    )
-from .repository import (
-    GitRepository,
-    LocalGitRepository,
-    GitRepositoryFormat,
-    )
-from .remote import (
-    RemoteGitRepository,
-    RemoteGitError,
-    )
-from .unpeel_map import (
-    UnpeelMap,
-    )
+
+EitherId = Tuple[Optional[RevisionID], Optional[ObjectID]]
+EitherRefDict = Dict[bytes, EitherId]
+RevidMap = Dict[RevisionID, Tuple[ObjectID, RevisionID]]
 
 
 class InterToGitRepository(InterRepository):
@@ -97,7 +59,7 @@ class InterToGitRepository(InterRepository):
     _matching_repo_format = GitRepositoryFormat()
 
     def __init__(self, source, target):
-        super(InterToGitRepository, self).__init__(source, target)
+        super().__init__(source, target)
         self.mapping = self.target.get_mapping()
         self.source_store = get_object_store(self.source, self.mapping)
 
@@ -105,11 +67,13 @@ class InterToGitRepository(InterRepository):
     def _get_repo_format_to_test():
         return None
 
-    def copy_content(self, revision_id=None, pb=None):
+    def copy_content(self, revision_id=None):
         """See InterRepository.copy_content."""
-        self.fetch(revision_id, pb, find_ghosts=False)
+        self.fetch(revision_id=revision_id, find_ghosts=False)
 
-    def fetch_refs(self, update_refs, lossy, overwrite=False):
+    def fetch_refs(
+            self, update_refs: Callable[[Dict[bytes, ObjectID]], Dict[bytes, ObjectID]],
+            lossy: bool, overwrite: bool = False) -> Tuple[RevidMap, Dict[bytes, ObjectID]]:
         """Fetch possibly roundtripped revisions into the target repository
         and update refs.
 
@@ -165,8 +129,10 @@ class InterToGitRepository(InterRepository):
 class InterToLocalGitRepository(InterToGitRepository):
     """InterBranch implementation between a Bazaar and a Git repository."""
 
+    target: LocalGitRepository
+
     def __init__(self, source, target):
-        super(InterToLocalGitRepository, self).__init__(source, target)
+        super().__init__(source, target)
         self.target_store = self.target.controldir._git.object_store
         self.target_refs = self.target.controldir._git.refs
 
@@ -229,7 +195,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                 pb.update("determining revisions to fetch", len(missing))
         return graph.iter_topo_order(missing)
 
-    def _get_target_bzr_refs(self):
+    def _get_target_either_refs(self) -> EitherRefDict:
         """Return a dictionary with references.
 
         :return: Dictionary with reference names as keys and tuples
@@ -256,11 +222,11 @@ class InterToLocalGitRepository(InterToGitRepository):
             bzr_refs[k] = (v, revid)
         return bzr_refs
 
-    def fetch_refs(self, update_refs, lossy, overwrite=False):
+    def fetch_refs(self, update_refs, lossy, overwrite: bool = False):
         self._warn_slow()
         result_refs = {}
         with self.source_store.lock_read():
-            old_refs = self._get_target_bzr_refs()
+            old_refs = self._get_target_either_refs()
             new_refs = update_refs(old_refs)
             revidmap = self.fetch_revs(
                 [(git_sha, bzr_revid)
@@ -286,7 +252,7 @@ class InterToLocalGitRepository(InterToGitRepository):
                     result_refs[name] = (gitid, revid if not lossy else self.mapping.revision_id_foreign_to_bzr(gitid))
         return revidmap, old_refs, result_refs
 
-    def fetch_revs(self, revs, lossy, limit=None):
+    def fetch_revs(self, revs, lossy: bool, limit: Optional[int] = None) -> RevidMap:
         if not lossy and not self.mapping.roundtripping:
             for git_sha, bzr_revid in revs:
                 if (bzr_revid is not None and
@@ -314,11 +280,9 @@ class InterToLocalGitRepository(InterToGitRepository):
                 self.target_store.add_objects(object_generator)
                 return revidmap
 
-    def fetch(self, revision_id=None, pb=None, find_ghosts=False,
-              fetch_spec=None, mapped_refs=None, lossy=False):
-        if mapped_refs is not None:
-            stop_revisions = mapped_refs
-        elif revision_id is not None:
+    def fetch(self, revision_id=None, find_ghosts: bool = False,
+              lossy=False, fetch_spec=None) -> FetchResult:
+        if revision_id is not None:
             stop_revisions = [(None, revision_id)]
         elif fetch_spec is not None:
             recipe = fetch_spec.get_recipe()
@@ -346,12 +310,15 @@ class InterToLocalGitRepository(InterToGitRepository):
 
 class InterToRemoteGitRepository(InterToGitRepository):
 
-    def fetch_refs(self, update_refs, lossy, overwrite=False):
+    target: RemoteGitRepository
+
+    def fetch_refs(self, update_refs, lossy, overwrite: bool = False):
         """Import the gist of the ancestry of a particular revision."""
         if not lossy and not self.mapping.roundtripping:
             raise NoPushSupport(self.source, self.target, self.mapping)
+
         unpeel_map = UnpeelMap.from_repository(self.source)
-        revidmap = {}
+        revidmap: Dict[bytes, bytes] = {}
 
         def git_update_refs(old_refs):
             ret = {}
@@ -373,14 +340,11 @@ class InterToRemoteGitRepository(InterToGitRepository):
         with self.source_store.lock_read():
             result = self.target.send_pack(
                 git_update_refs, self.source_store.generate_lossy_pack_data)
-            if result is not None and not isinstance(result, dict):
-                for ref, error in result.ref_status.items():
-                    if error:
-                        raise RemoteGitError(
-                            'unable to update ref %r: %s' % (ref, error))
-                new_refs = result.refs
-            else:  # dulwich < 0.20.3
-                new_refs = result
+            for ref, error in result.ref_status.items():
+                if error:
+                    raise RemoteGitError(
+                        'unable to update ref {!r}: {}'.format(ref, error))
+            new_refs = result.refs
         # FIXME: revidmap?
         return revidmap, self.old_refs, new_refs
 
@@ -391,7 +355,7 @@ class InterToRemoteGitRepository(InterToGitRepository):
                 isinstance(target, RemoteGitRepository))
 
 
-class GitSearchResult(object):
+class GitSearchResult(AbstractSearchResult):
 
     def __init__(self, start, exclude, keys):
         self._start = start
@@ -418,12 +382,12 @@ class InterFromGitRepository(InterRepository):
         def determine_wants(refs):
             unpeel_lookup = {}
             for k, v in refs.items():
-                if k.endswith(ANNOTATED_TAG_SUFFIX):
-                    unpeel_lookup[v] = refs[k[:-len(ANNOTATED_TAG_SUFFIX)]]
-            potential = set([unpeel_lookup.get(w, w) for w in wants])
+                if k.endswith(PEELED_TAG_SUFFIX):
+                    unpeel_lookup[v] = refs[k[:-len(PEELED_TAG_SUFFIX)]]
+            potential = {unpeel_lookup.get(w, w) for w in wants}
             if include_tags:
                 for k, sha in refs.items():
-                    if k.endswith(ANNOTATED_TAG_SUFFIX):
+                    if k.endswith(PEELED_TAG_SUFFIX):
                         continue
                     try:
                         tag_name = ref_to_tag_name(k)
@@ -487,7 +451,7 @@ class InterGitNonGitRepository(InterFromGitRepository):
                 continue
             else:
                 revids[revid] = sha
-        return set([revids[r] for r in self.target.has_revisions(revids)])
+        return {revids[r] for r in self.target.has_revisions(revids)}
 
     def determine_wants_all(self, refs):
         potential = set()
@@ -641,37 +605,51 @@ class InterLocalGitNonGitRepository(InterGitNonGitRepository):
 class InterGitGitRepository(InterFromGitRepository):
     """InterRepository that copies between Git repositories."""
 
-    def fetch_refs(self, update_refs, lossy, overwrite=False):
+    source: GitRepository
+    target: GitRepository
+
+    def _get_target_either_refs(self):
+        ret = {}
+        for name, sha1 in self.target.controldir.get_refs_container().as_dict().items():
+            ret[name] = (sha1, self.target.lookup_foreign_revision_id(sha1))
+        return ret
+
+    def fetch_refs(self, update_refs, lossy: bool = False, overwrite: bool = False) -> Tuple[RevidMap, EitherRefDict, EitherRefDict]:
         if lossy:
             raise LossyPushToSameVCS(self.source, self.target)
-        old_refs = self.target.controldir.get_refs_container()
+        old_refs = self._get_target_either_refs()
         ref_changes = {}
 
         def determine_wants(heads):
-            old_refs = dict([(k, (v, None))
-                             for (k, v) in heads.as_dict().items()])
+            old_refs = {k: (v, None)
+                             for (k, v) in heads.items()}
             new_refs = update_refs(old_refs)
+            ret = []
+            for name, (sha1, bzr_revid) in list(new_refs.items()):
+                if sha1 is None:
+                    sha1, unused_mapping = self.source.lookup_bzr_revision_id(bzr_revid)
+                new_refs[name] = (sha1, bzr_revid)
+                ret.append(sha1)
             ref_changes.update(new_refs)
-            return [sha1 for (sha1, bzr_revid) in new_refs.values()]
-        self.fetch_objects(determine_wants, lossy=lossy)
+            return ret
+        self.fetch_objects(determine_wants)
         for k, (git_sha, bzr_revid) in ref_changes.items():
-            self.target._git.refs[k] = git_sha
+            self.target._git.refs[k] = git_sha  # type: ignore
         new_refs = self.target.controldir.get_refs_container()
-        return None, old_refs, new_refs
+        return {}, old_refs, new_refs
 
-    def fetch_objects(self, determine_wants, mapping=None, limit=None,
-                      lossy=False):
+    def fetch_objects(self, determine_wants, limit=None, mapping=None, lossy=False):
         raise NotImplementedError(self.fetch_objects)
 
     def _target_has_shas(self, shas):
-        return set(
-            [sha for sha in shas if sha in self.target._git.object_store])
+        return {
+            sha for sha in shas if sha in self.target._git.object_store}
 
     def fetch(self, revision_id=None, find_ghosts=False,
-              mapping=None, fetch_spec=None, branches=None, limit=None,
+              fetch_spec=None, branches=None, limit=None,
               include_tags=False, lossy=False):
-        if mapping is None:
-            mapping = self.source.get_mapping()
+        if lossy:
+            raise LossyPushToSameVCS(self.source, self.target)
         if revision_id is not None:
             args = [revision_id]
         elif fetch_spec is not None:
@@ -691,7 +669,7 @@ class InterGitGitRepository(InterFromGitRepository):
             determine_wants = self.get_determine_wants_revids(
                 args, include_tags=include_tags)
         wants_recorder = DetermineWantsRecorder(determine_wants)
-        self.fetch_objects(wants_recorder, mapping, limit=limit, lossy=lossy)
+        self.fetch_objects(wants_recorder, limit=limit)
         result = FetchResult()
         result.refs = wants_recorder.remote_refs
         return result
@@ -712,7 +690,7 @@ class InterGitGitRepository(InterFromGitRepository):
                 if value == ZERO_SHA:
                     continue
 
-                if name.endswith(ANNOTATED_TAG_SUFFIX):
+                if name.endswith(PEELED_TAG_SUFFIX):
                     continue
 
                 if name in branches or (include_tags and is_tag(name)):
@@ -721,20 +699,22 @@ class InterGitGitRepository(InterFromGitRepository):
         return determine_wants
 
     def determine_wants_all(self, refs):
-        potential = set([
+        potential = {
             v for k, v in refs.items()
-            if not v == ZERO_SHA and not k.endswith(ANNOTATED_TAG_SUFFIX)])
+            if not v == ZERO_SHA and not k.endswith(PEELED_TAG_SUFFIX)}
         return list(potential - self._target_has_shas(potential))
 
 
 class InterLocalGitLocalGitRepository(InterGitGitRepository):
 
-    def fetch_objects(self, determine_wants, mapping=None, limit=None,
-                      lossy=False):
-        if lossy:
-            raise LossyPushToSameVCS(self.source, self.target)
+    source: LocalGitRepository
+    target: LocalGitRepository
+
+    def fetch_objects(self, determine_wants, limit=None, mapping=None, lossy: bool = False):
         if limit is not None:
             raise FetchLimitUnsupported(self)
+        if lossy:
+            raise LossyPushToSameVCS(self.source, self.target)
         from .remote import DefaultProgressReporter
         with ui.ui_factory.nested_progress_bar() as pb:
             progress = DefaultProgressReporter(pb).progress
@@ -752,10 +732,8 @@ class InterLocalGitLocalGitRepository(InterGitGitRepository):
 
 class InterRemoteGitLocalGitRepository(InterGitGitRepository):
 
-    def fetch_objects(self, determine_wants, mapping=None, limit=None,
-                      lossy=False):
-        if lossy:
-            raise LossyPushToSameVCS(self.source, self.target)
+    def fetch_objects(self, determine_wants, limit=None, mapping=None):
+        from tempfile import SpooledTemporaryFile
         if limit is not None:
             raise FetchLimitUnsupported(self)
         graphwalker = self.target._git.get_graph_walker()
@@ -763,12 +741,14 @@ class InterRemoteGitLocalGitRepository(InterGitGitRepository):
                 self.source.controldir._client._fetch_capabilities):
             # TODO(jelmer): Avoid reading entire file into memory and
             # only processing it after the whole file has been fetched.
-            f = BytesIO()
+            f = SpooledTemporaryFile(
+                max_size=PACK_SPOOL_FILE_MAX_SIZE, prefix='incoming-',
+                dir=getattr(self.target._git.object_store, 'path', None))
 
             def commit():
                 if f.tell():
                     f.seek(0)
-                    self.target._git.object_store.move_in_thin_pack(f)
+                    self.target._git.object_store.add_thin_pack(f.read, None)
 
             def abort():
                 pass
