@@ -24,34 +24,13 @@ from typing import Optional
 
 import fastbencode as bencode
 
-from .. import (
-    errors,
-    foreign,
-    trace,
-    urlutils,
-    )
-from ..foreign import (
-    ForeignVcs,
-    VcsMappingRegistry,
-    ForeignRevision,
-    )
-from ..revision import (
-    NULL_REVISION,
-    Revision,
-    )
-from .errors import (
-    NoPushSupport,
-    )
-from .hg import (
-    format_hg_metadata,
-    extract_hg_metadata,
-    )
-from .roundtrip import (
-    extract_bzr_metadata,
-    inject_bzr_metadata,
-    CommitSupplement,
-    )
-
+from .. import errors, foreign, trace, urlutils
+from ..foreign import ForeignRevision, ForeignVcs, VcsMappingRegistry
+from ..revision import NULL_REVISION, Revision
+from .errors import NoPushSupport
+from .hg import extract_hg_metadata, format_hg_metadata
+from .roundtrip import (CommitSupplement, extract_bzr_metadata,
+                        inject_bzr_metadata)
 
 DEFAULT_FILE_MODE = stat.S_IFREG | 0o644
 HG_RENAME_SOURCE = b"HG:rename-source"
@@ -277,10 +256,17 @@ class BzrGitMapping(foreign.VcsMapping):
         return message, metadata
 
     def _decode_commit_message(self, rev, message, encoding):
-        return message.decode(encoding), CommitSupplement()
+        if message is None:
+            decoded_message = None
+        else:
+            decoded_message = message.decode(encoding)
+        return decoded_message, CommitSupplement()
 
     def _encode_commit_message(self, rev, message, encoding):
-        return message.encode(encoding)
+        if message is None:
+            return None
+        else:
+            return message.encode(encoding)
 
     def export_commit(self, rev, tree_sha, parent_lookup, lossy,
                       verifiers):
@@ -382,10 +368,14 @@ class BzrGitMapping(foreign.VcsMapping):
                 Tag.from_string(rev.properties[propname].encode('utf-8', 'surrogateescape')))
             i += 1
             propname = 'git-mergetag-%d' % i
+        try:
+            extra = commit._extra
+        except AttributeError:
+            extra = commit.extra
         if 'git-extra' in rev.properties:
             for l in rev.properties['git-extra'].splitlines():
                 (k, v) = l.split(' ', 1)
-                commit.extra.append(
+                extra.append(
                     (k.encode('utf-8', 'surrogateescape'),
                      v.encode('utf-8', 'surrogateescape')))
         return commit
@@ -395,14 +385,15 @@ class BzrGitMapping(foreign.VcsMapping):
             encoding = commit.encoding.decode('ascii')
         else:
             encoding = 'utf-8'
-        try:
-            message, metadata = self._decode_commit_message(
-                None, commit.message, encoding)
-        except UnicodeDecodeError:
-            pass
-        else:
-            if metadata.revision_id:
-                return metadata.revision_id
+        if commit.message is not None:
+            try:
+                message, metadata = self._decode_commit_message(
+                    None, commit.message, encoding)
+            except UnicodeDecodeError:
+                pass
+            else:
+                if metadata.revision_id:
+                    return metadata.revision_id
         return self.revision_id_foreign_to_bzr(commit.id)
 
     def import_commit(self, commit, lookup_parent_revid, strict=True):
@@ -483,7 +474,11 @@ class BzrGitMapping(foreign.VcsMapping):
             rev.parent_ids = list(parents)
         unknown_extra_fields = []
         extra_lines = []
-        for k, v in commit.extra:
+        try:
+            extra = commit._extra
+        except AttributeError:
+            extra = commit.extra
+        for k, v in extra:
             if k == HG_RENAME_SOURCE:
                 extra_lines.append(
                     k.decode('utf-8', 'surrogateescape') + ' ' +
