@@ -17,64 +17,34 @@
 
 """Git Trees."""
 
-from collections import deque
 import errno
+import os
+import posixpath
+import stat
+from collections import deque
 from functools import partial
 from io import BytesIO
-import os
 
-from dulwich.config import (
-    parse_submodules,
-    ConfigFile as GitConfigFile,
-    )
-from dulwich.diff_tree import tree_changes, RenameDetector
+from dulwich.config import ConfigFile as GitConfigFile
+from dulwich.config import parse_submodules
+from dulwich.diff_tree import RenameDetector, tree_changes
 from dulwich.errors import NotTreeError
-from dulwich.index import (
-    blob_from_path_and_stat,
-    cleanup_mode,
-    commit_tree,
-    index_entry_from_stat,
-    Index,
-    IndexEntry,
-    )
-from dulwich.object_store import (
-    OverlayObjectStore,
-    )
-from dulwich.objects import (
-    Blob,
-    Tree,
-    ZERO_SHA,
-    S_IFGITLINK,
-    S_ISGITLINK,
-    )
-import stat
-import posixpath
+from dulwich.index import (Index, IndexEntry, blob_from_path_and_stat,
+                           cleanup_mode, commit_tree, index_entry_from_stat)
+from dulwich.object_store import OverlayObjectStore, iter_tree_contents
+from dulwich.objects import S_IFGITLINK, S_ISGITLINK, ZERO_SHA, Blob, Tree
 
-from .. import (
-    controldir as _mod_controldir,
-    delta,
-    errors,
-    mutabletree,
-    osutils,
-    revisiontree,
-    trace,
-    transport as _mod_transport,
-    tree as _mod_tree,
-    workingtree,
-    )
-from ..revision import (
-    CURRENT_REVISION,
-    NULL_REVISION,
-    )
-
-from .mapping import (
-    encode_git_path,
-    decode_git_path,
-    mode_is_executable,
-    mode_kind,
-    default_mapping,
-    )
+from .. import controldir as _mod_controldir
+from .. import delta, errors, mutabletree, osutils, revisiontree, trace
+from .. import transport as _mod_transport
+from .. import tree as _mod_tree
+from .. import urlutils, workingtree
 from ..bzr.inventorytree import InventoryTreeChange
+from ..revision import CURRENT_REVISION, NULL_REVISION
+from ..transport import get_transport
+from ..tree import MissingNestedTree
+from .mapping import (decode_git_path, default_mapping, encode_git_path,
+                      mode_is_executable, mode_kind)
 
 
 class GitTreeDirectory(_mod_tree.TreeDirectory):
@@ -99,7 +69,7 @@ class GitTreeDirectory(_mod_tree.TreeDirectory):
             self.file_id, self.name, self.parent_id)
 
     def __repr__(self):
-        return "%s(file_id=%r, name=%r, parent_id=%r)" % (
+        return "{}(file_id={!r}, name={!r}, parent_id={!r})".format(
             self.__class__.__name__, self.file_id, self.name,
             self.parent_id)
 
@@ -176,7 +146,7 @@ class GitTreeSymlink(_mod_tree.TreeLink):
         return None
 
     def __repr__(self):
-        return "%s(file_id=%r, name=%r, parent_id=%r, symlink_target=%r)" % (
+        return "{}(file_id={!r}, name={!r}, parent_id={!r}, symlink_target={!r})".format(
             type(self).__name__, self.file_id, self.name, self.parent_id,
             self.symlink_target)
 
@@ -258,6 +228,10 @@ class GitTree(_mod_tree.Tree):
 
     supports_file_ids = False
 
+    @classmethod
+    def is_special_path(cls, path):
+        return path.startswith('.git')
+
     def supports_symlinks(self):
         return True
 
@@ -329,6 +303,11 @@ class GitTree(_mod_tree.Tree):
         return Branch.open(url.decode('utf-8'))
 
 
+class RemoteNestedTree(MissingNestedTree):
+
+    _fmt = "Unable to access remote nested tree at %(path)s"
+
+
 class GitRevisionTree(revisiontree.RevisionTree, GitTree):
     """Revision tree implementation based on Git objects."""
 
@@ -358,22 +337,37 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
         if not isinstance(relpath, bytes):
             raise TypeError(relpath)
         try:
-            info = self._submodule_info()[relpath]
+            url, section = self._submodule_info()[relpath]
         except KeyError:
+            nested_repo_transport = None
+        else:
+            nested_repo_transport = self._repository.controldir.control_transport.clone(
+                posixpath.join('modules', decode_git_path(section)))
+            if not nested_repo_transport.has('.'):
+                nested_url = urlutils.join(
+                    self._repository.controldir.user_url, decode_git_path(url))
+                nested_repo_transport = get_transport(nested_url)
+        if nested_repo_transport is None:
             nested_repo_transport = self._repository.controldir.user_transport.clone(
                 decode_git_path(relpath))
         else:
             nested_repo_transport = self._repository.controldir.control_transport.clone(
-                posixpath.join('modules', decode_git_path(info[1])))
+                posixpath.join('modules', decode_git_path(section)))
             if not nested_repo_transport.has('.'):
                 nested_repo_transport = self._repository.controldir.user_transport.clone(
-                    posixpath.join(decode_git_path(info[1]), '.git'))
-        nested_controldir = _mod_controldir.ControlDir.open_from_transport(
-            nested_repo_transport)
+                    posixpath.join(decode_git_path(section), '.git'))
+        try:
+            nested_controldir = _mod_controldir.ControlDir.open_from_transport(
+                nested_repo_transport)
+        except errors.NotBranchError as e:
+            raise MissingNestedTree(decode_git_path(relpath)) from e
         return nested_controldir.find_repository()
 
     def _get_submodule_store(self, relpath):
-        return self._get_submodule_repository(relpath)._git.object_store
+        repo = self._get_submodule_repository(relpath)
+        if not hasattr(repo, '_git'):
+            raise RemoteNestedTree(relpath)
+        return repo._git.object_store
 
     def get_nested_tree(self, path):
         encoded_path = encode_git_path(path)
@@ -415,7 +409,7 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
         return self.mapping.generate_file_id(osutils.safe_unicode(path))
 
     def all_versioned_paths(self):
-        ret = {u''}
+        ret = {''}
         todo = [(self.store, b'', self.tree)]
         while todo:
             (store, path, tree_id) = todo.pop()
@@ -480,7 +474,7 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
         if self.tree is None:
             return
         if from_dir is None or from_dir == '.':
-            from_dir = u""
+            from_dir = ""
         (store, mode, hexsha) = self._lookup_path(from_dir)
         if mode is None:  # Root
             root_ie = self._get_dir_ie(b"", None)
@@ -575,11 +569,11 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
             if specific_files in ([""], []):
                 specific_files = None
             else:
-                specific_files = set([encode_git_path(p)
-                                      for p in specific_files])
+                specific_files = {encode_git_path(p)
+                                      for p in specific_files}
         todo = deque([(self.store, b"", self.tree, self.path2id(''))])
-        if specific_files is None or u"" in specific_files:
-            yield u"", self._get_dir_ie(b"", None)
+        if specific_files is None or "" in specific_files:
+            yield "", self._get_dir_ie(b"", None)
         while todo:
             store, path, tree_sha, parent_id = todo.popleft()
             tree = store[tree_sha]
@@ -590,9 +584,13 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
                 child_path = posixpath.join(path, name)
                 child_path_decoded = decode_git_path(child_path)
                 if recurse_nested and S_ISGITLINK(mode):
-                    mode = stat.S_IFDIR
-                    substore = self._get_submodule_store(child_path)
-                    hexsha = substore[hexsha].tree
+                    try:
+                        substore = self._get_submodule_store(child_path)
+                    except errors.NotBranchError:
+                        substore = store
+                    else:
+                        mode = stat.S_IFDIR
+                        hexsha = substore[hexsha].tree
                 else:
                     substore = store
                 if stat.S_ISDIR(mode):
@@ -659,7 +657,7 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
         if S_ISGITLINK(mode):
             try:
                 nested_repo = self._get_submodule_repository(encode_git_path(path))
-            except errors.NotBranchError:
+            except MissingNestedTree:
                 return self.mapping.revision_id_foreign_to_bzr(hexsha)
             else:
                 try:
@@ -698,8 +696,8 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
     def _iter_tree_contents(self, include_trees=False):
         if self.tree is None:
             return iter([])
-        return self.store.iter_tree_contents(
-            self.tree, include_trees=include_trees)
+        return iter_tree_contents(
+            self.store, self.tree, include_trees=include_trees)
 
     def annotate_iter(self, path, default_revision=CURRENT_REVISION):
         """Return an iterator of revision_id, line tuples.
@@ -714,6 +712,7 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
         with self.lock_read():
             # Now we have the parents of this content
             from breezy.annotate import Annotator
+
             from .annotate import AnnotateProvider
             annotator = Annotator(AnnotateProvider(
                 self._repository._file_change_scanner))
@@ -725,7 +724,7 @@ class GitRevisionTree(revisiontree.RevisionTree, GitTree):
     def _get_rules_searcher(self, default_searcher):
         return default_searcher
 
-    def walkdirs(self, prefix=u""):
+    def walkdirs(self, prefix=""):
         (store, mode, hexsha) = self._lookup_path(prefix)
         todo = deque(
             [(store, encode_git_path(prefix), hexsha)])
@@ -802,7 +801,7 @@ def tree_delta_from_git_changes(changes, mappings,
                 oldkind = None
             if oldpath == b'':
                 oldparent = None
-                oldname = u''
+                oldname = ''
             else:
                 (oldparentpath, oldname) = osutils.split(oldpath_decoded)
                 oldparent = old_mapping.generate_file_id(oldparentpath)
@@ -820,9 +819,9 @@ def tree_delta_from_git_changes(changes, mappings,
             else:
                 newexe = False
                 newkind = None
-            if newpath_decoded == u'':
+            if newpath_decoded == '':
                 newparent = None
-                newname = u''
+                newname = ''
             else:
                 newparentpath, newname = osutils.split(newpath_decoded)
                 newparent = new_mapping.generate_file_id(newparentpath)
@@ -944,9 +943,9 @@ def changes_from_git_changes(changes, mapping, specific_files=None,
             else:
                 oldexe = False
                 oldkind = None
-            if oldpath_decoded == u'':
+            if oldpath_decoded == '':
                 oldparent = None
-                oldname = u''
+                oldname = ''
             else:
                 (oldparentpath, oldname) = osutils.split(oldpath_decoded)
                 oldparent = mapping.generate_file_id(oldparentpath)
@@ -964,9 +963,9 @@ def changes_from_git_changes(changes, mapping, specific_files=None,
             else:
                 newexe = False
                 newkind = None
-            if newpath_decoded == u'':
+            if newpath_decoded == '':
                 newparent = None
-                newname = u''
+                newname = ''
             else:
                 newparentpath, newname = osutils.split(newpath_decoded)
                 newparent = mapping.generate_file_id(newparentpath)
@@ -996,12 +995,10 @@ def changes_from_git_changes(changes, mapping, specific_files=None,
 class InterGitTrees(_mod_tree.InterTree):
     """InterTree that works between two git trees."""
 
-    _matching_from_tree_format = None
-    _matching_to_tree_format = None
     _test_mutable_trees_to_test_trees = None
 
     def __init__(self, source, target):
-        super(InterGitTrees, self).__init__(source, target)
+        super().__init__(source, target)
         if self.source.store == self.target.store:
             self.store = self.source.store
         else:
@@ -1294,7 +1291,7 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
             blob = Blob()
             try:
                 file, stat_val = self.get_file_with_stat(path)
-            except (_mod_transport.NoSuchFile, IOError):
+            except (_mod_transport.NoSuchFile, OSError):
                 # TODO: Rather than come up with something here, use the old
                 # index
                 file = BytesIO()
@@ -1310,7 +1307,7 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
             blob = Blob()
             try:
                 stat_val = self._lstat(path)
-            except EnvironmentError:
+            except OSError:
                 # TODO: Rather than come up with something here, use the
                 # old index
                 stat_val = os.stat_result(
@@ -1332,7 +1329,7 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
                     raise errors.NoCommits(path)
             try:
                 stat_val = self._lstat(path)
-            except EnvironmentError:
+            except OSError:
                 stat_val = os.stat_result(
                     (S_IFGITLINK, 0, 0, 0, 0, 0, 0, 0, 0, 0))
             stat_val = os.stat_result((S_IFGITLINK, ) + stat_val[1:])
@@ -1364,10 +1361,9 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
             for path, value in index.items():
                 if S_ISGITLINK(value.mode) and recurse_nested:
                     subindex = self._get_submodule_index(path)
-                    for entry in self._recurse_index_entries(
+                    yield from self._recurse_index_entries(
                             index=subindex, basepath=path,
-                            recurse_nested=recurse_nested):
-                        yield entry
+                            recurse_nested=recurse_nested)
                 else:
                     yield (posixpath.join(basepath, path), value)
 
@@ -1378,11 +1374,11 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
                 specific_files = set(specific_files)
             else:
                 specific_files = None
-            root_ie = self._get_dir_ie(u"", None)
+            root_ie = self._get_dir_ie("", None)
             ret = {}
-            if specific_files is None or u"" in specific_files:
-                ret[(u"", u"")] = root_ie
-            dir_ids = {u"": root_ie.file_id}
+            if specific_files is None or "" in specific_files:
+                ret[("", "")] = root_ie
+            dir_ids = {"": root_ie.file_id}
             for path, value in self._recurse_index_entries(
                     recurse_nested=recurse_nested):
                 if self.mapping.is_special_file(path):
@@ -1407,7 +1403,8 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
                     key = (posixpath.dirname(path), path)
                     if key not in ret and self.is_versioned(path):
                         ret[key] = self._get_dir_ie(path, self.path2id(key[0]))
-            return ((path, ie) for ((_, path), ie) in sorted(ret.items()))
+            for ((_, path), ie) in sorted(ret.items()):
+                yield path, ie
 
     def iter_references(self):
         if self.supports_tree_reference():
@@ -1618,7 +1615,11 @@ class MutableGitIndexTree(mutabletree.MutableTree, GitTree):
             raise
         kind = mode_kind(stat_result.st_mode)
         if kind == 'file':
-            return self._file_content_summary(path, stat_result)
+            size = stat_result.st_size
+            executable = self._is_executable_from_path_and_stat(path, stat_result)
+            # try for a stat cache lookup
+            return ('file', size, executable, self._sha_from_stat(
+                path, stat_result))
         elif kind == 'directory':
             # perhaps it looks like a plain directory, but it's really a
             # reference.
@@ -1719,7 +1720,7 @@ def snapshot_workingtree(target, want_unversioned=False):
     for path, index_entry in target._recurse_index_entries():
         try:
             live_entry = target._live_entry(path)
-        except EnvironmentError as e:
+        except OSError as e:
             if e.errno == errno.ENOENT:
                 # Entry was removed; keep it listed, but mark it as gone.
                 blobs[path] = (ZERO_SHA, 0)
