@@ -14,27 +14,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from __future__ import absolute_import
-
 import subprocess
 import tempfile
 
-from ... import (
-    errors,
-    revision as _mod_revision,
-    )
-from ...config import (
-    ListOption,
-    Option,
-    bool_from_store,
-    int_from_store,
-    )
-
-from ...smtp_connection import SMTPConnection
+from ... import errors
+from ... import revision as _mod_revision
+from ...config import ListOption, Option, bool_from_store, int_from_store
 from ...email_message import EmailMessage
+from ...smtp_connection import SMTPConnection
 
 
-class EmailSender(object):
+class EmailSender:
     """An email message sender."""
 
     _smtplib_implementation = SMTPConnection
@@ -80,7 +70,7 @@ class EmailSender(object):
 
         # We must use StringIO.StringIO because we want a Unicode string that
         # we can pass to send_email and have that do the proper encoding.
-        from ...sixish import StringIO
+        from io import StringIO
         outf = StringIO()
 
         _body = self.config.get('post_commit_body')
@@ -122,6 +112,7 @@ class EmailSender(object):
             return
 
         from ...diff import show_diff_trees
+
         # optionally show the diff if its smaller than the post_commit_difflimit option
         revid_new = self.revision.revision_id
         if self.revision.parent_ids:
@@ -137,7 +128,7 @@ class EmailSender(object):
 
         # We can use a StringIO because show_diff_trees should only write
         # 8-bit strings. It is an error to write a Unicode string here.
-        from ...sixish import StringIO
+        from io import StringIO
         diff_content = StringIO()
         diff_options = self.config.get('post_commit_diffoptions')
         show_diff_trees(tree_old, tree_new, diff_content, None, diff_options)
@@ -227,7 +218,7 @@ class EmailSender(object):
             rc = process.wait()
             if rc != 0:
                 raise errors.BzrError(
-                    "Failed to send email: exit status %s" % (rc,))
+                    "Failed to send email: exit status {}".format(rc))
 
     def _send_using_smtplib(self):
         """Use python's smtplib to send the email."""
@@ -275,7 +266,7 @@ class EmailSender(object):
         return self._format(_subject)
 
     def diff_filename(self):
-        return "patch-%s.diff" % (self.revno,)
+        return "patch-{}.diff".format(self.revno)
 
 
 opt_post_commit_body = Option("post_commit_body",
