@@ -16,35 +16,27 @@
 
 __docformat__ = "google"
 
-from typing import List, Type, TYPE_CHECKING
+from typing import List, Type, TYPE_CHECKING, Optional, Iterable
 
 from .lazy_import import lazy_import
+
 lazy_import(globals(), """
 import time
 
 from breezy import (
     config,
-    debug,
-    graph,
     osutils,
-    revision as _mod_revision,
     )
 from breezy.i18n import gettext
 """)
 
-from . import (
-    controldir,
-    errors,
-    registry,
-    ui,
-    )
+from . import controldir, debug, errors, graph, registry, revision as _mod_revision, ui
 from .decorators import only_raises
 from .inter import InterObject
-from .lock import _RelockDebugMixin, LogicalLockResult
-from .trace import (
-    log_exception_quietly, note, mutter, mutter_callsite, warning)
+from .lock import LogicalLockResult, _RelockDebugMixin
 from .revisiontree import RevisionTree
-
+from .trace import (log_exception_quietly, mutter, mutter_callsite, note,
+                    warning)
 
 # Old formats display a warning, but only once
 _deprecation_warning_done = False
@@ -931,6 +923,7 @@ class Repository(controldir.ControlComponent, _RelockDebugMixin):
           children are included.
         """
         from .tree import InterTree
+
         # Get the revision-ids of interest
         required_trees = set()
         for revision in revisions:
@@ -1451,92 +1444,49 @@ class RepositoryFormat(controldir.ControlComponentFormat):
             hook(params)
 
 
-# formats which have no format string are not discoverable or independently
-# creatable on disk, so are not registered in format_registry.  They're
-# all in breezy.bzr.knitreponow.  When an instance of one of these is
-# needed, it's constructed directly by the ControlDir.  Non-native formats where
-# the repository is not separately opened are similar.
+class AbstractSearchResult:
+    """The result of a search, describing a set of keys.
 
-format_registry.register_lazy(
-    b'Bazaar-NG Knit Repository Format 1',
-    'breezy.bzr.knitrepo',
-    'RepositoryFormatKnit1',
-    )
+    Search results are typically used as the 'fetch_spec' parameter when
+    fetching revisions.
 
-format_registry.register_lazy(
-    b'Bazaar Knit Repository Format 3 (bzr 0.15)\n',
-    'breezy.bzr.knitrepo',
-    'RepositoryFormatKnit3',
-    )
+    :seealso: AbstractSearch
+    """
 
-format_registry.register_lazy(
-    b'Bazaar Knit Repository Format 4 (bzr 1.0)\n',
-    'breezy.bzr.knitrepo',
-    'RepositoryFormatKnit4',
-    )
+    def get_recipe(self):
+        """Return a recipe that can be used to replay this search.
 
-# Pack-based formats. There is one format for pre-subtrees, and one for
-# post-subtrees to allow ease of testing.
-# NOTE: These are experimental in 0.92. Stable in 1.0 and above
-format_registry.register_lazy(
-    b'Bazaar pack repository format 1 (needs bzr 0.92)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack1',
-    )
-format_registry.register_lazy(
-    b'Bazaar pack repository format 1 with subtree support (needs bzr 0.92)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack3',
-    )
-format_registry.register_lazy(
-    b'Bazaar pack repository format 1 with rich root (needs bzr 1.0)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack4',
-    )
-format_registry.register_lazy(
-    b'Bazaar RepositoryFormatKnitPack5 (bzr 1.6)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack5',
-    )
-format_registry.register_lazy(
-    b'Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6.1)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack5RichRoot',
-    )
-format_registry.register_lazy(
-    b'Bazaar RepositoryFormatKnitPack5RichRoot (bzr 1.6)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack5RichRootBroken',
-    )
-format_registry.register_lazy(
-    b'Bazaar RepositoryFormatKnitPack6 (bzr 1.9)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack6',
-    )
-format_registry.register_lazy(
-    b'Bazaar RepositoryFormatKnitPack6RichRoot (bzr 1.9)\n',
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatKnitPack6RichRoot',
-    )
-format_registry.register_lazy(
-    b'Bazaar repository format 2a (needs bzr 1.16 or later)\n',
-    'breezy.bzr.groupcompress_repo',
-    'RepositoryFormat2a',
-    )
+        The recipe allows reconstruction of the same results at a later date.
 
-# Development formats.
-# Check their docstrings to see if/when they are obsolete.
-format_registry.register_lazy(
-    (b"Bazaar development format 2 with subtree support "
-        b"(needs bzr.dev from before 1.8)\n"),
-    'breezy.bzr.knitpack_repo',
-    'RepositoryFormatPackDevelopment2Subtree',
-    )
-format_registry.register_lazy(
-    b'Bazaar development format 8\n',
-    'breezy.bzr.groupcompress_repo',
-    'RepositoryFormat2aSubtree',
-    )
+        :return: A tuple of `(search_kind_str, *details)`.  The details vary by
+            kind of search result.
+        """
+        raise NotImplementedError(self.get_recipe)
+
+    def get_network_struct(self):
+        """Return a tuple that can be transmitted via the HPSS protocol."""
+        raise NotImplementedError(self.get_network_struct)
+
+    def get_keys(self):
+        """Return the keys found in this search.
+
+        :return: A set of keys.
+        """
+        raise NotImplementedError(self.get_keys)
+
+    def is_empty(self):
+        """Return false if the search lists 1 or more revisions."""
+        raise NotImplementedError(self.is_empty)
+
+    def refine(self, seen, referenced):
+        """Create a new search by refining this search.
+
+        :param seen: Revisions that have been satisfied.
+        :param referenced: Revision references observed while satisfying some
+            of this search.
+        :return: A search result.
+        """
+        raise NotImplementedError(self.refine)
 
 
 class InterRepository(InterObject[Repository]):
@@ -1554,7 +1504,7 @@ class InterRepository(InterObject[Repository]):
     _optimisers = []
     """The available optimised InterRepository types."""
 
-    def copy_content(self, revision_id=None):
+    def copy_content(self, revision_id: Optional[_mod_revision.RevisionID] = None) -> None:
         """Make a complete copy of the content in self into destination.
 
         This is a destructive operation! Do not use it on existing
@@ -1572,7 +1522,7 @@ class InterRepository(InterObject[Repository]):
                 pass
             self.target.fetch(self.source, revision_id=revision_id)
 
-    def fetch(self, revision_id=None, find_ghosts=False, lossy=False):
+    def fetch(self, revision_id: Optional[_mod_revision.RevisionID] = None, find_ghosts: bool = False, lossy: bool = False) -> FetchResult:
         """Fetch the content required to construct revision_id.
 
         The content is copied from self.source to self.target.
@@ -1585,8 +1535,9 @@ class InterRepository(InterObject[Repository]):
         raise NotImplementedError(self.fetch)
 
     def search_missing_revision_ids(
-            self, find_ghosts=True, revision_ids=None, if_present_ids=None,
-            limit=None):
+            self, find_ghosts: bool = True, revision_ids: Optional[Iterable[_mod_revision.RevisionID]] = None,
+            if_present_ids: Optional[Iterable[_mod_revision.RevisionID]] = None,
+            limit: Optional[int] = None) -> AbstractSearchResult:
         """Return the revision ids that source has that target does not.
 
         Args:
@@ -1601,7 +1552,7 @@ class InterRepository(InterObject[Repository]):
             rather than just finding the surface difference.
           limit: Maximum number of revisions to return, topologically
             ordered
-        Returns: A breezy.graph.SearchResult.
+        Returns: A SearchResult.
         """
         raise NotImplementedError(self.search_missing_revision_ids)
 
