@@ -17,22 +17,12 @@
 
 import os
 
-from breezy import (
-    branch,
-    conflicts,
-    controldir,
-    errors,
-    mutabletree,
-    osutils,
-    revision as _mod_revision,
-    tests,
-    transport as _mod_transport,
-    ui,
-    )
-from breezy.commit import (
-    CannotCommitSelectedFileMerge,
-    PointlessCommit,
-    )
+from breezy import branch, conflicts, controldir, errors, mutabletree, osutils
+from breezy import revision as _mod_revision
+from breezy import tests
+from breezy import transport as _mod_transport
+from breezy import ui
+from breezy.commit import CannotCommitSelectedFileMerge, PointlessCommit
 from breezy.tests.matchers import HasPathRelations
 from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 from breezy.tests.testui import ProgressRecordingUIFactory
