@@ -15,71 +15,36 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 import errno
-from io import BytesIO
 import os
 import sys
 import time
-
-from ... import (
-    osutils,
-    tests,
-    trace,
-    transform,
-    urlutils,
-    )
-from ...transport import FileExists
-from ...tree import TreeChange
-from ...bzr.conflicts import (
-    DeletingParent,
-    DuplicateEntry,
-    DuplicateID,
-    MissingParent,
-    NonDirectoryParent,
-    ParentLoop,
-    UnversionedParent,
-)
-from ...errors import (
-    DuplicateKey,
-    ExistingLimbo,
-    ExistingPendingDeletion,
-    ImmortalPendingDeletion,
-    LockError,
-)
-from ...osutils import (
-    file_kind,
-    pathjoin,
-)
-from .. import (
-    features,
-    TestSkipped,
-    )
-from ..features import (
-    HardlinkFeature,
-    SymlinkFeature,
-    )
-from ...transform import (
-    create_from_tree,
-    FinalPaths,
-    resolve_conflicts,
-    ROOT_PARENT,
-    ImmortalLimbo,
-    MalformedTransform,
-    NoFinalPath,
-    ReusingTransform,
-    TransformRenameFailed,
-)
+from io import BytesIO
 
 from breezy.bzr.transform import resolve_checkout
-
-from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 from breezy.tests.matchers import MatchesTreeChanges
+from breezy.tests.per_workingtree import TestCaseWithWorkingTree
 
+from ... import osutils, tests, trace, transform, urlutils
+from ...bzr.conflicts import (DeletingParent, DuplicateEntry, DuplicateID,
+                              MissingParent, NonDirectoryParent, ParentLoop,
+                              UnversionedParent)
+from ...errors import (DuplicateKey, ExistingLimbo, ExistingPendingDeletion,
+                       ImmortalPendingDeletion, LockError)
+from ...osutils import file_kind, pathjoin
+from ...transform import (ROOT_PARENT, FinalPaths, ImmortalLimbo,
+                          MalformedTransform, NoFinalPath, ReusingTransform,
+                          TransformRenameFailed, create_from_tree,
+                          resolve_conflicts)
+from ...transport import FileExists
+from ...tree import TreeChange
+from .. import TestSkipped, features
+from ..features import HardlinkFeature, SymlinkFeature
 
 
 class TestTreeTransform(TestCaseWithWorkingTree):
 
     def setUp(self):
-        super(TestTreeTransform, self).setUp()
+        super().setUp()
         self.wt = self.make_branch_and_tree('wt')
 
     def transform(self):
@@ -261,7 +226,7 @@ class TestTreeTransform(TestCaseWithWorkingTree):
         # We only guarantee 2s resolution
         self.assertTrue(
             abs(creation_mtime - st1.st_mtime) < 2.0,
-            "%s != %s within 2 seconds" % (creation_mtime, st1.st_mtime))
+            "{} != {} within 2 seconds".format(creation_mtime, st1.st_mtime))
         # But if we have more than that, all files should get the same result
         self.assertEqual(st1.st_mtime, st2.st_mtime)
 
@@ -407,7 +372,7 @@ class TestTreeTransform(TestCaseWithWorkingTree):
         self.assertEqual(len(transform.find_raw_conflicts()), 0)
         transform.apply()
         self.assertRaises(ReusingTransform, transform.find_raw_conflicts)
-        with open(self.wt.abspath('name'), 'r') as f:
+        with open(self.wt.abspath('name')) as f:
             self.assertEqual('contents', f.read())
         self.assertIs(self.wt.is_executable('name'), True)
         self.assertTrue(self.wt.is_versioned('name'))
@@ -718,19 +683,19 @@ class TestTreeTransform(TestCaseWithWorkingTree):
             self.assertEqual(mangle_tree.final_file_id(mfile2), b'mfile2')
         self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
         mangle_tree.apply()
-        with open(self.wt.abspath('name1'), 'r') as f:
+        with open(self.wt.abspath('name1')) as f:
             self.assertEqual(f.read(), 'hello2')
-        with open(self.wt.abspath('name2'), 'r') as f:
+        with open(self.wt.abspath('name2')) as f:
             self.assertEqual(f.read(), 'hello1')
         mfile2_path = self.wt.abspath(pathjoin('new_directory', 'mfile2'))
         self.assertEqual(mangle_tree.final_parent(mfile2), newdir)
-        with open(mfile2_path, 'r') as f:
+        with open(mfile2_path) as f:
             self.assertEqual(f.read(), 'later2')
         if self.wt.supports_setting_file_ids():
             self.assertEqual(self.wt.id2path(b'mfile2'), 'new_directory/mfile2')
             self.assertEqual(self.wt.path2id('new_directory/mfile2'), b'mfile2')
         newfile_path = self.wt.abspath(pathjoin('new_directory', 'newfile'))
-        with open(newfile_path, 'r') as f:
+        with open(newfile_path) as f:
             self.assertEqual(f.read(), 'hello3')
         if self.wt.supports_setting_file_ids():
             self.assertEqual(self.wt.path2id('dying_directory'), b'ddir')
@@ -857,10 +822,10 @@ class TestTreeTransform(TestCaseWithWorkingTree):
 
     def test_symlinks_unicode(self):
         self.requireFeature(features.UnicodeFilenameFeature)
-        self._test_symlinks(u'\N{Euro Sign}wizard',
-                            u'wizard-targ\N{Euro Sign}t',
-                            u'\N{Euro Sign}wizard2',
-                            u'b\N{Euro Sign}hind_curtain')
+        self._test_symlinks('\N{Euro Sign}wizard',
+                            'wizard-targ\N{Euro Sign}t',
+                            '\N{Euro Sign}wizard2',
+                            'b\N{Euro Sign}hind_curtain')
 
     def test_unsupported_symlink_no_conflict(self):
         def tt_helper():
@@ -947,8 +912,8 @@ class TestTreeTransform(TestCaseWithWorkingTree):
             self.assertEqual(len(cooked_conflicts), 7)
         else:
             self.assertEqual(
-                set([c.path for c in cooked_conflicts]),
-                set(['oz/emeraldcity', 'oz', 'munchkincity', 'dorothy.moved']))
+                {c.path for c in cooked_conflicts},
+                {'oz/emeraldcity', 'oz', 'munchkincity', 'dorothy.moved'})
         tt.finalize()
 
     def test_string_conflicts(self):
@@ -984,7 +949,7 @@ class TestTreeTransform(TestCaseWithWorkingTree):
                  'Text conflict in munchkincity',
                  'Text conflict in oz',
                  'Text conflict in oz/emeraldcity'},
-                set([c for c in conflicts_s]))
+                {c for c in conflicts_s})
 
     def prepare_wrong_parent_kind(self):
         tt, root = self.transform()
@@ -1399,12 +1364,12 @@ class TestTreeTransform(TestCaseWithWorkingTree):
             transform.set_executability(True, transform.trans_id_tree_path('file2'))
             self.assertTreeChanges(transform, [
                 TreeChange(
-                    (u'file1', u'file1'), True, (True, True),
-                    ('file1', u'file1'),
+                    ('file1', 'file1'), True, (True, True),
+                    ('file1', 'file1'),
                     ('file', None), (False, False), False),
                 TreeChange(
-                    (u'file2', u'file2'), False, (True, True),
-                    ('file2', u'file2'),
+                    ('file2', 'file2'), False, (True, True),
+                    ('file2', 'file2'),
                     ('file', 'file'), (False, True), False)])
         finally:
             transform.finalize()
